@@ -3,6 +3,7 @@
 *************************************************/
 
 /* Copyright (c) University of Cambridge 1995 - 2018 */
+/* Copyright (c) The Exim Maintainers 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This file, which provides support for Microsoft's Secure Password
@@ -46,13 +47,13 @@ References:
 
 optionlist auth_spa_options[] = {
   { "client_domain",             opt_stringptr,
-      (void *)(offsetof(auth_spa_options_block, spa_domain)) },
+      OPT_OFF(auth_spa_options_block, spa_domain) },
   { "client_password",           opt_stringptr,
-      (void *)(offsetof(auth_spa_options_block, spa_password)) },
+      OPT_OFF(auth_spa_options_block, spa_password) },
   { "client_username",           opt_stringptr,
-      (void *)(offsetof(auth_spa_options_block, spa_username)) },
+      OPT_OFF(auth_spa_options_block, spa_username) },
   { "server_password",           opt_stringptr,
-      (void *)(offsetof(auth_spa_options_block, spa_serverpassword)) }
+      OPT_OFF(auth_spa_options_block, spa_serverpassword) }
 };
 
 /* Size of the options list. An extern variable has to be used so that its
@@ -139,42 +140,35 @@ SPAAuthChallenge challenge;
 SPAAuthResponse  response;
 SPAAuthResponse  *responseptr = &response;
 uschar msgbuf[2048];
-uschar *clearpass;
+uschar *clearpass, *s;
+unsigned off;
 
 /* send a 334, MS Exchange style, and grab the client's request,
 unless we already have it via an initial response. */
 
-if ((*data == '\0') &&
-    (auth_get_no64_data(&data, US"NTLM supported") != OK))
-  {
-  /* something borked */
+if (!*data && auth_get_no64_data(&data, US"NTLM supported") != OK)
   return FAIL;
-  }
 
-if (spa_base64_to_bits(CS (&request), sizeof(request), CCS (data)) < 0)
+if (spa_base64_to_bits(CS &request, sizeof(request), CCS data) < 0)
   {
   DEBUG(D_auth) debug_printf("auth_spa_server(): bad base64 data in "
-  "request: %s\n", data);
+    "request: %s\n", data);
   return FAIL;
   }
 
 /* create a challenge and send it back */
 
-spa_build_auth_challenge(&request,&challenge);
-spa_bits_to_base64 (msgbuf, (unsigned char*)&challenge,
-    spa_request_length(&challenge));
+spa_build_auth_challenge(&request, &challenge);
+spa_bits_to_base64(msgbuf, US &challenge, spa_request_length(&challenge));
 
 if (auth_get_no64_data(&data, msgbuf) != OK)
-  {
-  /* something borked */
   return FAIL;
-  }
 
 /* dump client response */
-if (spa_base64_to_bits(CS (&response), sizeof(response), CCS (data)) < 0)
+if (spa_base64_to_bits(CS &response, sizeof(response), CCS data) < 0)
   {
   DEBUG(D_auth) debug_printf("auth_spa_server(): bad base64 data in "
-  "response: %s\n", data);
+    "response: %s\n", data);
   return FAIL;
   }
 
@@ -194,9 +188,19 @@ that causes failure if the size of msgbuf is exceeded. ****/
 
   {
   int i;
-  char *p = ((char*)responseptr) + IVAL(&responseptr->uUser.offset,0);
+  char * p;
   int len = SVAL(&responseptr->uUser.len,0)/2;
 
+  if (  (off = IVAL(&responseptr->uUser.offset,0)) >= sizeof(SPAAuthResponse)
+     || len >= sizeof(responseptr->buffer)/2
+     || (p = (CS responseptr) + off) + len*2 >= CS (responseptr+1)
+     )
+    {
+    DEBUG(D_auth)
+      debug_printf("auth_spa_server(): bad uUser spec in response\n");
+    return FAIL;
+    }
+
   if (len + 1 >= sizeof(msgbuf)) return FAIL;
   for (i = 0; i < len; ++i)
     {
@@ -221,9 +225,7 @@ debug_print_string(ablock->server_debug_string);    /* customized debug */
 
 /* look up password */
 
-clearpass = expand_string(ob->spa_serverpassword);
-if (clearpass == NULL)
-  {
+if (!(clearpass = expand_string(ob->spa_serverpassword)))
   if (f.expand_string_forcedfail)
     {
     DEBUG(D_auth) debug_printf("auth_spa_server(): forced failure while "
@@ -236,22 +238,25 @@ if (clearpass == NULL)
       "spa_serverpassword: %s\n", expand_string_message);
     return DEFER;
     }
-  }
 
 /* create local hash copy */
 
-spa_smb_encrypt (clearpass, challenge.challengeData, lmRespData);
-spa_smb_nt_encrypt (clearpass, challenge.challengeData, ntRespData);
+spa_smb_encrypt(clearpass, challenge.challengeData, lmRespData);
+spa_smb_nt_encrypt(clearpass, challenge.challengeData, ntRespData);
 
 /* compare NT hash (LM may not be available) */
 
-if (memcmp(ntRespData,
-      ((unsigned char*)responseptr)+IVAL(&responseptr->ntResponse.offset,0),
-      24) == 0)
-  /* success. we have a winner. */
+off = IVAL(&responseptr->ntResponse.offset,0);
+if (off >= sizeof(SPAAuthResponse) - 24)
   {
-  return auth_check_serv_cond(ablock);
+  DEBUG(D_auth)
+    debug_printf("auth_spa_server(): bad ntRespData spec in response\n");
+  return FAIL;
   }
+s = (US responseptr) + off;
+
+if (memcmp(ntRespData, s, 24) == 0)
+  return auth_check_serv_cond(ablock);	/* success. we have a winner. */
 
   /* Expand server_condition as an authorization check (PH) */
 
@@ -325,9 +330,8 @@ if (!smtp_read_response(sx, US buffer, buffsize, '3', timeout))
 
 DSPA("\n\n%s authenticator: using domain %s\n\n", ablock->name, domain);
 
-spa_build_auth_request (&request, CS username, domain);
-spa_bits_to_base64 (US msgbuf, (unsigned char*)&request,
-       spa_request_length(&request));
+spa_build_auth_request(&request, CS username, domain);
+spa_bits_to_base64(US msgbuf, US &request, spa_request_length(&request));
 
 DSPA("\n\n%s authenticator: sending request (%s)\n\n", ablock->name, msgbuf);
 
@@ -341,11 +345,10 @@ if (!smtp_read_response(sx, US buffer, buffsize, '3', timeout))
 
 /* convert the challenge into the challenge struct */
 DSPA("\n\n%s authenticator: challenge (%s)\n\n", ablock->name, buffer + 4);
-spa_base64_to_bits (CS (&challenge), sizeof(challenge), CCS (buffer + 4));
+spa_base64_to_bits(CS (&challenge), sizeof(challenge), CCS (buffer + 4));
 
-spa_build_auth_response (&challenge, &response, CS username, CS password);
-spa_bits_to_base64 (US msgbuf, (unsigned char*)&response,
-       spa_request_length(&response));
+spa_build_auth_response(&challenge, &response, CS username, CS password);
+spa_bits_to_base64(US msgbuf, US &response, spa_request_length(&response));
 DSPA("\n\n%s authenticator: challenge response (%s)\n\n", ablock->name, msgbuf);
 
 /* send the challenge response */
