@@ -1951,7 +1951,7 @@ while (commands)
 	  (long int)geteuid());
 	if (log_fd < 0)
 	  {
-	  if (log_filename == NULL)
+	  if (!log_filename)
 	    {
 	    *error_pointer = US"attempt to obey \"logwrite\" command "
 	      "without a previous \"logfile\"";
@@ -1960,7 +1960,7 @@ while (commands)
 	  log_fd = Uopen(log_filename, O_CREAT|O_APPEND|O_WRONLY, log_mode);
 	  if (log_fd < 0)
 	    {
-	    *error_pointer = string_open_failed(errno, "filter log file \"%s\"",
+	    *error_pointer = string_open_failed("filter log file \"%s\"",
 	      log_filename);
 	    return FF_ERROR;
 	    }
@@ -1974,9 +1974,8 @@ while (commands)
 	  }
 	}
       else
-	{
-	DEBUG(D_filter) debug_printf_indent("skipping logwrite (verifying or testing)\n");
-	}
+	DEBUG(D_filter)
+	  debug_printf_indent("skipping logwrite (verifying or testing)\n");
       break;
 
       /* Header addition and removal is available only in the system filter. The
@@ -2007,11 +2006,9 @@ while (commands)
 	else if (subtype == FALSE)
 	  {
 	  int sep = 0;
-	  uschar *ss;
-	  const uschar *list = s;
-	  uschar buffer[128];
-	  while ((ss = string_nextinlist(&list, &sep, buffer, sizeof(buffer)))
-		 != NULL)
+	  const uschar * list = s;
+
+	  for (uschar * ss; ss = string_nextinlist(&list, &sep, NULL, 0); )
 	    header_remove(0, ss);
 	  }
 
