@@ -59,7 +59,7 @@ if (rblock->extra_headers)
       shared with other addresses. The output function outputs them in reverse
       order. */
 
-      header_line *  h = store_get(sizeof(header_line));
+      header_line *  h = store_get(sizeof(header_line), FALSE);
 
       /* We used to use string_sprintf() to add the newline if needed, but that
       causes problems if the header line is exceedingly long (e.g. adding
@@ -69,7 +69,7 @@ if (rblock->extra_headers)
 	h->text = s;
       else
 	{
-	h->text = store_get(slen+2);
+	h->text = store_get(slen+2, is_tainted(s));
 	memcpy(h->text, s, slen);
 	h->text[slen++] = '\n';
 	h->text[slen] = 0;
