@@ -518,7 +518,7 @@ typedef struct address_item_propagated {
   uschar *remove_headers;         /* list of those to remove */
   void   *variables;		  /* router-vasriables */
 
-#ifdef EXPERIMENTAL_SRS
+#ifdef EXPERIMENTAL_SRS_ALT
   uschar *srs_sender;             /* Change return path when delivering */
 #endif
   BOOL    ignore_error:1;	  /* ignore delivery error */
@@ -626,6 +626,8 @@ typedef struct address_item {
     BOOL af_verify_routed:1;		/* for cached sender verify: routed OK */
     BOOL af_verify_callout:1;		/* for cached sender verify: callout was specified */
     BOOL af_include_affixes:1;		/* delivered with affixes in RCPT */
+    BOOL af_new_conn:1;			/* delivered on an fresh TCP conn */
+    BOOL af_cont_conn:1;		/* delivered (with new MAIL cmd) on an existing TCP conn */
     BOOL af_cert_verified:1;		/* delivered with verified TLS cert */
     BOOL af_pass_message:1;		/* pass message in bounces */
     BOOL af_bad_reply:1;		/* filter could not generate autoreply */
@@ -647,7 +649,7 @@ typedef struct address_item {
 #ifdef SUPPORT_I18N
     BOOL af_utf8_downcvt:1;		/* downconvert was done for delivery */
 #endif
-#ifdef EXPERIMENTAL_TLS_RESUME
+#ifndef DISABLE_TLS_RESUME
     BOOL af_tls_resume:1;		/* TLS used a resumed session */
 #endif
   } flags;
